package com.admin.system.domain

import com.fasterxml.jackson.annotation.JsonIgnore
import com.admin.security.domain.BaseModel
import com.admin.system.service.dto.RoleSmallDTO
import com.admin.system.service.dto.SysUserDTO
import org.springframework.beans.BeanUtils
import java.util.*
import javax.persistence.*
import javax.validation.constraints.NotBlank

@Entity
@Table(name = "sys_role")
open class SysRole : BaseModel() {


    @Column(nullable = false)
    @NotBlank
    val name: String? = null

    // 数据权限类型 全部 、 本级 、 自定义
    @Column(name = "data_scope")
    val dataScope = "本级"

    // 数值越小，级别越大
    @Column(name = "level")
    val level = 3

    @Column
    val remark: String? = null

    @JsonIgnore
    @ManyToMany(mappedBy = "roles", targetEntity = SysUser::class)
    val users: List<SysUser>? = null

    @ManyToMany
    @JoinTable(name = "sys_roles_permissions", joinColumns = [JoinColumn(name = "role_id", referencedColumnName = "id")], inverseJoinColumns = [JoinColumn(name = "permission_id", referencedColumnName = "id")])
    val permissions: List<SysPermission>? = null

    @ManyToMany
    @JoinTable(name = "sys_roles_menus", joinColumns = [JoinColumn(name = "role_id", referencedColumnName = "id")], inverseJoinColumns = [JoinColumn(name = "menu_id", referencedColumnName = "id")])
    val menus: List<SysMenu>? = null

    @ManyToMany
    @JoinTable(name = "sys_roles_depts", joinColumns = [JoinColumn(name = "role_id", referencedColumnName = "id")], inverseJoinColumns = [JoinColumn(name = "dept_id", referencedColumnName = "id")])
    val depts: List<SysDept>? = null


    override fun toString(): String {
        return "Role{" +
                "id=" + id +
                ", name='" + name + '\''.toString() +
                ", remark='" + remark + '\''.toString() +
                '}'.toString()
    }

    override fun equals(o: Any?): Boolean {
        if (this === o) return true
        if (o == null || javaClass != o.javaClass) return false
        val role = o as SysRole?
        return id == role!!.id
    }

    override fun hashCode(): Int {
        return Objects.hash(id)
    }

    fun convertDto(): RoleSmallDTO {
        val dto = RoleSmallDTO()
        BeanUtils.copyProperties(this, dto)
        return dto
    }

}
