package com.admin.system.domain

import com.admin.security.domain.BaseModel
import com.admin.system.service.dto.SysUserDTO
import org.springframework.beans.BeanUtils
import java.time.LocalDateTime
import javax.persistence.*
import javax.validation.constraints.NotBlank
import javax.validation.constraints.NotNull
import javax.validation.constraints.Pattern

@Entity
@Table(name = "sys_user")
open class SysUser : BaseModel() {

    @NotBlank
    @Column(unique = true)
    var username: String? = null

    var name: String? = null

    var avatar: String? = null

    @NotBlank
    @Pattern(regexp = "([a-z0-9A-Z]+[-|.]?)+[a-z0-9A-Z]@([a-z0-9A-Z]+(-[a-z0-9A-Z]+)?\\.)+[a-zA-Z]{2,}", message = "格式错误")
    var email: String? = null

    @NotBlank
    var phone: String? = null

    @NotNull
    var enabled: Boolean? = null

    var password: String? = null

    @Column(name = "last_password_reset_time")
    var lastPasswordResetTime: LocalDateTime? = null

    @ManyToMany
//    @JoinTable(name = "users_roles",
    @JoinTable(name = "sys_users_roles",
            joinColumns = [JoinColumn(name = "user_id", referencedColumnName = "id")],
            inverseJoinColumns = [JoinColumn(name = "role_id", referencedColumnName = "id")])
    var roles: List<SysRole>? = null

    @OneToOne
    @JoinColumn(name = "job_id")
    var job: SysJob? = null

    @OneToOne
    @JoinColumn(name = "dept_id")
    var dept: SysDept? = null

    fun convertDto(): SysUserDTO {
        var dto = SysUserDTO()
        BeanUtils.copyProperties(this, dto)
        dto.dept = this.dept?.convertDto()
        dto.roles = this.roles?.map { it.convertDto() }?.toList()
        dto.job = this.job?.convertDto()
        return dto
    }

}